pub mod prime_utils {
    use num::{BigUint, One};

    #[must_use]
    pub fn log_2(x: &BigUint) -> u64 {
        x.bits() - 1
    }

    #[must_use]
    pub fn is_prime(number: &BigUint, g_primes: &Vec<BigUint>) -> bool {
        if BigUint::from(1u8) == *number {
            return false;
        }
        if BigUint::from(4u8) > *number {
            return true;
        }

        // number = 2^a - 1
        // a = log2(number + 1)
        let a = log_2(&(number + 1u8));

        //if number isn't a mersenne number
        if BigUint::from(2u8).pow(a as u32) - BigUint::one() != *number {
            const ZERO: BigUint = BigUint::ZERO;
            let mut i = BigUint::one();
            let one = BigUint::one();

            let sqrtnum = number.sqrt() + &one; //fake ceil function

            for prime in g_primes {
                if prime < &sqrtnum && number % prime == ZERO {
                    return false;
                }
            }

            loop {
                i += &one;
                if number % &i == ZERO {
                    return false;
                }
                if i == sqrtnum {
                    return true;
                }
            }
        }

        //check if it's a mersenne prime
        // 4 12 194
        let mut last = BigUint::from(4u8);
        let two = BigUint::from(2u8);

        for _i in 2..a {
            last = (last.pow(2) - &two) % number;
        }

        last == BigUint::from(0u8)
    }
}
